%% Add project directories to Matlab's path
functionPath = fileparts(mfilename('fullpath')); % Directory of the current function
addpath(genpath([functionPath filesep '..']));

defineFeatureTypes();

%==========================================================================
%% Example 1

inputFile = 'SynthDB-3-002.avi';
resultPath = './Results';

maxNFrames = +inf; % Process the whole video

% Extract image features
extractColor(inputFile, [], maxNFrames, false, true);
speeds = [0 : 0.1 : 1.0 +inf]; % Slow speeds and flicker
extractHoMEs(inputFile, [], [1], speeds, maxNFrames, true, 'G2H2', true);

% Define segmentation parameters
p.initialColorBasedSegmentation = true;
p.featureTypes = [COLOR HOMES];
p.featuresToUse = {[1], [1], [1], [1], [1], [2]}; % Use color for a few levels, then HoMEs
p.mergingPercentiles = [25 3];
p.connectivity = 26;
p.nHierarchyLevels = 90;
p.minSegmentDuration = +inf;
p.minSegmentArea = 10;
p.minNSegments = 4;
p.maxNFrames = +inf;
p.saveVideos = true;

nLevels = segmentVideo(inputFile, resultPath, p);
visualizeSegmentation(resultPath, nLevels); % Display the last highest level of the segmentation (with just 2 segments remaining in this case)

%==========================================================================
%% Example 2

inputFile = 'rocking_horse.avi';
resultPath = './Results';
maxNFrames = +inf; % Process the whole video

% Extract image features
extractColor(inputFile, 0.5, maxNFrames, false);
speeds = [0 : 0.1 : 1 +inf]; % Slow speeds + flicker
extractHoMEs(inputFile, 0.5, [1], speeds, maxNFrames, false, 'G2H2', true);

p.initialColorBasedSegmentation = true;
p.featureTypes = [COLOR HOMES];
p.featuresToUse = {[1], [1 2]}; % Use color for the first level, then color+HoMEs
p.mergingPercentiles = [25 3];
p.connectivity = 26;
p.nHierarchyLevels = 90;
p.minSegmentDuration = +inf; % Enforce segments to last for the whole video
p.minSegmentArea = 10;
p.minNSegments = 3;
p.maxNFrames = +inf;
p.saveVideos = true;

segmentVideo(inputFile, resultPath, p);
makeBoundaryMap(resultPath, false);

%==========================================================================
%% Example 3

inputFile = 'ocean-fire.avi';
resultPath = './Results';
maxNFrames = 20; % Only process the first few frames of the video (faster !)

% Extract image features
extractColor(inputFile, 0.5, maxNFrames, false);
speeds = [0 : 0.25 : 1.0 +inf]; % Slow speeds and flicker
extractHoMEs(inputFile, 0.5, [1], speeds, maxNFrames, false, 'G2H2', true);

% Define segmentation parameters
p.initialColorBasedSegmentation = true;
p.featureTypes = [COLOR HOMES];
p.featuresToUse = {[1], [1 2]}; % Use color for the first level, then color+HoMEs
p.mergingPercentiles = [25 3];
p.connectivity = 26;
p.nHierarchyLevels = 90;
p.minSegmentDuration = +inf; % Enforce segments to last for the whole video
p.minSegmentArea = 10;
p.minNSegments = 2;
p.maxNFrames = +inf;
p.saveVideos = true;

nLevels = segmentVideo(inputFile, resultPath, p);
visualizeSegmentation(resultPath, nLevels); % Display the last highest level of the segmentation (with just 2 segments remaining in this case)

%==========================================================================
%% Clean up files containing image features (large files !)

delete('*_features*.mat')
