function result = convolveBasis3D(input, basis, is)
%CONVOLVEBASIS Convolve input with separate basis functions in each dimension.

% Author: Damien Teney

% Rename
basisX = basis(is(1), :);
basisY = basis(is(2), :);
basisZ = basis(is(3), :);

if isempty(input)
  % No input signal was given: just return the basis functions
  result = basisY' ... % Y dimension
         * basisX;     % X dimension
  result = repmat(result, [1 1 length(basisZ)]) ... % Z dimension
        .* repmat(reshape(basisZ, 1, 1, length(basisZ)), [length(basisY) length(basisX) 1]);
else
  % Convolve the given signal with the basis functions
  result = input;
  result = imfilter(result, reshape(basisX, 1, []   ), 'symmetric', 'same', 'conv'); % X dimension
  result = imfilter(result, reshape(basisY, [], 1   ), 'symmetric', 'same', 'conv'); % Y dimension
  result = imfilter(result, reshape(basisZ, 1, 1, []), 'symmetric', 'same', 'conv'); % Z dimension
end
