function [ G3Basis filterHalfSize ] = init3DG3(input, sigma)
%INIT... Compute convolutions of input with basis functions for steerable filters.

% Author: Damien Teney, inspired by code from Konstatinos G. Derpanis

% Define basis functions
if nargin < 2, sigma = 1.0; end % Missing argument: set default value
filterHalfSize = 6 * sigma;
t = [-filterHalfSize : filterHalfSize] ./ (2 * sigma);
C = 0.184;
basis(1, :) = -4 * C * (2 * t.^3 - 3*t) .* exp(-t.^2);
basis(2, :) = t .* exp(-t.^2);
basis(3, :) = -4 * C * (2*t.^2 - 1) .* exp(-t.^2);
basis(4, :) = exp(-t.^2);
basis(5, :) = -8 * C * t .* exp(-t.^2);

% Convolve the input with the basis
G3Basis{1}  = convolveBasis3D(input, basis, [1 4 4]);
G3Basis{2}  = convolveBasis3D(input, basis, [3 2 4]);
G3Basis{3}  = convolveBasis3D(input, basis, [2 3 4]);
G3Basis{4}  = convolveBasis3D(input, basis, [4 1 4]);
G3Basis{5}  = convolveBasis3D(input, basis, [3 4 2]);
G3Basis{6}  = convolveBasis3D(input, basis, [5 2 2]);
G3Basis{7}  = convolveBasis3D(input, basis, [4 3 2]);
G3Basis{8}  = convolveBasis3D(input, basis, [2 4 3]);
G3Basis{9}  = convolveBasis3D(input, basis, [4 2 3]);
G3Basis{10} = convolveBasis3D(input, basis, [4 4 1]);
