function [ G4Basis filterHalfSize ] = init3DG4(input, sigma)
%INIT... Compute convolutions of input with basis functions for steerable filters.

% Author: Damien Teney, inspired by code from Konstatinos G. Derpanis

% Define basis functions
if nargin < 2, sigma = 1.0; end % Missing argument: set default value
filterHalfSize = 6 * sigma;
t = [-filterHalfSize : filterHalfSize] ./ (2 * sigma);
C = 0.0696;
basis(1, :) = C * (16*t.^4 - 48*t.^2 + 12) .* exp(-t.^2);
basis(2, :) = C * (16*t.^3 - 24*t) .* exp(-t.^2);
basis(3, :) = 2 * sqrt(C) * (-2*t.^2 + 1) .* exp(-t.^2);
basis(4, :) = C * (16 * t.^2 - 8) .* exp(-t.^2);
basis(5, :) = exp(-t.^2);
basis(6, :) = t .* exp(-t.^2);

% Convolve the input with the basis
G4Basis{1}  = convolveBasis3D(input, basis, [1 5 5]);
G4Basis{2}  = convolveBasis3D(input, basis, [2 6 5]);
G4Basis{3}  = convolveBasis3D(input, basis, [3 3 5]);
G4Basis{4}  = convolveBasis3D(input, basis, [6 2 5]);
G4Basis{5}  = convolveBasis3D(input, basis, [5 1 5]);
G4Basis{6}  = convolveBasis3D(input, basis, [2 5 6]);
G4Basis{7}  = convolveBasis3D(input, basis, [4 6 6]);
G4Basis{8}  = convolveBasis3D(input, basis, [6 4 6]);
G4Basis{9}  = convolveBasis3D(input, basis, [5 2 6]);
G4Basis{10} = convolveBasis3D(input, basis, [3 5 3]);
G4Basis{11} = convolveBasis3D(input, basis, [6 6 4]);
G4Basis{12} = convolveBasis3D(input, basis, [5 3 3]);
G4Basis{13} = convolveBasis3D(input, basis, [6 5 2]);
G4Basis{14} = convolveBasis3D(input, basis, [5 6 2]);
G4Basis{15} = convolveBasis3D(input, basis, [5 5 1]);
