function [ H2Basis filterHalfSize ] = init3DH2(input, sigma)
%INIT... Compute convolutions of input with basis functions for steerable filters.

% Author: Damien Teney, inspired by code from Konstatinos G. Derpanis

% Define basis functions
if nargin < 2, sigma = 1.0; end % Missing argument: set default value
filterHalfSize = 4 * sigma;
t = [-filterHalfSize : filterHalfSize] ./ (sqrt(2) * sigma);
C = 0.877776;
basis(1, :) = C * (t.^3 - 2.254 * t) .* exp(-t.^2); 
basis(2, :) = C * (t.^2 - 0.751333) .* exp(-t.^2); 
basis(3, :) = exp(-t.^2); 
basis(4, :) = C * t .* exp(-t.^2);
basis(5, :) = t .* exp(-t.^2);

% Convolve the input with the basis
H2Basis{1}  = convolveBasis3D(input, basis, [1 3 3]);
H2Basis{2}  = convolveBasis3D(input, basis, [2 5 3]);
H2Basis{3}  = convolveBasis3D(input, basis, [5 2 3]);
H2Basis{4}  = convolveBasis3D(input, basis, [3 1 3]);
H2Basis{5}  = convolveBasis3D(input, basis, [2 3 5]);
H2Basis{6}  = convolveBasis3D(input, basis, [4 5 5]);
H2Basis{7}  = convolveBasis3D(input, basis, [3 2 5]);
H2Basis{8}  = convolveBasis3D(input, basis, [5 3 2]);
H2Basis{9}  = convolveBasis3D(input, basis, [3 5 2]);
H2Basis{10}  = convolveBasis3D(input, basis, [3 3 1]);
