function [ H3Basis filterHalfSize ] = init3DH3(input, sigma)
%INIT... Compute convolutions of input with basis functions for steerable filters.

% Author: Damien Teney, inspired by code from Konstatinos G. Derpanis

% Define basis functions
if nargin < 2, sigma = 1.0; end % Missing argument: set default value
filterHalfSize = 6 * sigma;
t = [-filterHalfSize : filterHalfSize] ./ (2 * sigma);
C = 0.89448;
basis(1, :) = C * (-0.9454 + 2.959*t.^2 - 0.6582*t.^4) .* exp(-t.^2);
basis(2, :) = C * (1.4795*t - 0.6582*t.^3) .* exp(-t.^2);
basis(3, :) = sqrt(C) .* (-sqrt(0.6582)*t.^2 + 0.7585*sqrt(0.6582)) .* exp(-t.^2);
basis(4, :) = sqrt(C) .* ( sqrt(0.6582)*t.^2 - 0.7585*sqrt(0.6582)) .* exp(-t.^2);
basis(5, :) = C .* (0.493168 - 0.6582*t.^2) .* exp(-t.^2);
basis(6, :) = t .* exp(-t.^2);
basis(7, :) = exp(-t.^2);

% Equate the sums of positive and negative components (separately), so that the integral of filter is 0 so that it does not respond to constant input, and also so that its integral does not vary with its orientation
%%{
for i = 1:size(basis, 1)
  tmp = basis(i, :);
  tmp(tmp > 0) = tmp(tmp > 0) / abs(sum(tmp(tmp > 0)));
  tmp(tmp < 0) = tmp(tmp < 0) / abs(sum(tmp(tmp < 0)));
  basis(i, :) = tmp;
end
%}

% Debug display
%{
figure(); hold on;
for i = 1:size(basis, 1)
  subplot(size(basis, 1), 1, i);
  plot(basis(i, :));
end
%}

% Convolve the input with the basis
H3Basis{1}  = convolveBasis3D(input, basis, [1 7 7]);
H3Basis{2}  = convolveBasis3D(input, basis, [2 6 7]);
H3Basis{3}  = convolveBasis3D(input, basis, [3 4 7]);
H3Basis{4}  = convolveBasis3D(input, basis, [6 2 7]);
H3Basis{5}  = convolveBasis3D(input, basis, [7 1 7]);
H3Basis{6}  = convolveBasis3D(input, basis, [2 7 6]);
H3Basis{7}  = convolveBasis3D(input, basis, [5 6 6]);
H3Basis{8}  = convolveBasis3D(input, basis, [6 5 6]);
H3Basis{9}  = convolveBasis3D(input, basis, [7 2 6]);
H3Basis{10} = convolveBasis3D(input, basis, [3 7 4]);
H3Basis{11} = convolveBasis3D(input, basis, [6 6 5]);
H3Basis{12} = convolveBasis3D(input, basis, [7 3 4]);
H3Basis{13} = convolveBasis3D(input, basis, [6 7 2]);
H3Basis{14} = convolveBasis3D(input, basis, [7 6 2]);
H3Basis{15} = convolveBasis3D(input, basis, [7 7 1]);
