function G2Basis = initG2(input)
%INIT... Compute convolutions of input with basis functions for steerable filters.

% Author: Damien Teney, inspired by code from Konstatinos G. Derpanis

% Define basis functions
f1 = [ 0.0094  0.1148  0.3964 -0.0601 -0.9213 -0.0601 0.3964 0.1148 0.0094];
f2 = [ 0.0008  0.0176  0.1660  0.6383  1.0     0.6383 0.1660 0.0176 0.0008];
f3 = [-0.0028 -0.0480 -0.3020 -0.5806  0.0     0.5806 0.3020 0.0480 0.0028];

if isempty(input)
  % No input signal was given: just return the basis functions
  G2Basis{1} = f2' * f1;
  G2Basis{2} = f3' * f3;
  G2Basis{3} = f1' * f2;
else
  % Convolve the given signal with the basis functions
  G2Basis{1} = conv2(f2, f1, input, 'same');
  G2Basis{2} = conv2(f3, f3, input, 'same');
  G2Basis{3} = conv2(f1, f2, input, 'same');
end
