function H2Basis = initH2(input)
%INIT... Compute convolutions of input with basis functions for steerable filters.

% Author: Damien Teney, inspired by code from Konstatinos G. Derpanis

% Define basis functions
f1 = [-0.0098  -0.0618  0.0998  0.7551  0.0    -0.7551  -0.0998  0.0618  0.0098];
f2 = [ 0.0008   0.0176  0.1660  0.6383  1.0     0.6383   0.1660  0.0176  0.0008];
f3 = [-0.0020  -0.0354 -0.2225 -0.4277  0.0     0.4277   0.2225  0.0354  0.0020];
f4 = [ 0.0048   0.0566  0.1695 -0.1889 -0.7349 -0.1889   0.1695  0.0566  0.0048];

if isempty(input)
  % No input signal was given: just return the basis functions
  H2Basis{1} = f2' * f1;
  H2Basis{2} = f3' * f4;
  H2Basis{3} = f4' * f3;
  H2Basis{4} = f1' * f2;
else
  % Convolve the given signal with the basis functions
  H2Basis{1} = conv2(f2, f1, input, 'same');
  H2Basis{2} = conv2(f3, f4, input, 'same');
  H2Basis{3} = conv2(f4, f3, input, 'same');
  H2Basis{4} = conv2(f1, f2, input, 'same');
end
