function bigImage = assembleImages(images)
%ASSEMBLEIMAGES Assemble images next to each other to make a giant image.
%   ASSEMBLEIMAGES(IMAGES) assembles the given images (cell array of NxM
%   matrices of the same size).

%   Author: Damien Teney

nImages = length(images);
if nImages == 0
  bigImage = []
  return;
end
h = size(images{1}, 1);
w = size(images{1}, 2);

n = ceil(nImages / ceil(sqrt(nImages)));
m = ceil(sqrt(nImages));
bigImage = zeros(m * h, n * w, size(images{1}, 3), class(images{1}));

is = 1:(1 + h - 1);
js = 1:(1 + w - 1);
for imageId = 1:nImages
  if size(images{imageId}, 1) ~= h ...
  || size(images{imageId}, 2) ~= w ...
    error('All images must be of the same size');
  end

  for channel = 1:size(bigImage, 3)
    bigImage(is, js, channel) = images{imageId}(:, :, channel);
  end

  if js(end) + w <= size(bigImage, 2)
    js = js + w;
  else
    % New row
    is = is + h;
    js = 1:(1 + w - 1); % Reinitialize
  end
end
