function flowPolar = flowUvToPolar(flowUV)
%FLOWUVTOPOLAR Convert a matrix of optical flow of a video from UV to polar (angle/magnitude) representation.

%   Author: Damien Teney

% Check input dimensions
assert(ndims(flowPolar) == 4);
assert(size(flowPolar, 4) == 2);

% Rename
flowU = flowUV(:, :, :, 1); % Horizontal flow
flowV = flowUV(:, :, :, 2); % Vertical flow

% Get flow magnitudes
flowMagnitudes = sqrt(flowU .^ 2 + flowV .^ 2);

% Get flow angles
flowU = flowU ./ flowMagnitudes;
flowV = flowV ./ flowMagnitudes;
flowAngles = atan2(flowV, flowU);
flowAngles = mod(flowAngles, 2 * pi);
flowAngles(flowMagnitudes == 0) = 0; % Default angle when the magnitude is 0
assert(all(flowAngles(:) >= 0) && all(flowAngles(:) < 2 * pi));

flowPolar = cat(4, flowAngles, flowMagnitudes);
