//=========================================================================
// incrementArrayInPlace(A, INDICES, INCREMENTVALUES) increments
// the elements of A such that
// A(INDICES(i)) = A(INDICES(i)) + INCREMENTVALUES(i)
//
// If an index appears several times, the corresponding element is
// accordingly incremented multiple times.
//
// For example:
//   a = [0 0 0];
//   incrementArrayInPlace(a, [1 3 1 1], [1 5 1 1]) % Now: a == [3 0 5]
//
// All arguments must be of the type 'double'.
//
// Author: Damien Teney
//=========================================================================

#include <mex.h>

//=========================================================================
// Input arguments:
//   a(...) vector, matrix or N-D array; any dimensions; type: double
//   indices(nIndices) type double
//   incrementValues(nIndices) type double
// Output arguments:
//   none, 'a' is modified in-place
//=========================================================================
void mexFunction(int nlhs, mxArray *plhs[],
                 int nrhs, const mxArray *prhs[]) {
  int i;
  // Input
  double *a, *indices, *incrementValues;
  int nElements, nIndices, nIncrementValues;

  // Check the number of arguments
  if (nrhs < 3)
    mexErrMsgTxt ("Wrong number of input arguments !");

  // Get input
  nIndices         = (int) mxGetM(prhs[1]) * (int) mxGetN(prhs[1]);
  nIncrementValues = (int) mxGetM(prhs[2]) * (int) mxGetN(prhs[2]);
  if (nIndices != nIncrementValues)
    mexErrMsgTxt ("INDICES and INCREMENTVALUES must be of the same size !");
  if (((int) mxGetM(prhs[1]) != 1) && ((int) mxGetN(prhs[1]) != 1))
    mexErrMsgTxt ("INDICES must be a vector (1xN or Nx1) !");
  if (((int) mxGetM(prhs[2]) != 1) && ((int) mxGetN(prhs[2]) != 1))
    mexErrMsgTxt ("INCREMENTVALUES must be a vector (1xN or Nx1) !");
  if (mxGetClassID(prhs[0]) != mxDOUBLE_CLASS
   || mxGetClassID(prhs[1]) != mxDOUBLE_CLASS
   || mxGetClassID(prhs[2]) != mxDOUBLE_CLASS)
    mexErrMsgTxt ("Invalid data types !");
  a               = (double *) mxGetPr(prhs[0]);
  indices         = (double *) mxGetPr(prhs[1]);
  incrementValues = (double *) mxGetPr(prhs[2]);

  // Get the total number of elements in 'a'
  nElements = (int) mxGetNumberOfElements(prhs[0]);

  // Increment values
  for (i = 0; i < nIndices; i++) {
    // Error checking (slow)
    //if (indices[i] < 1 || indices[i] > nElements)
    //  mexErrMsgTxt("Invalid indices !");
    a[(int) indices[i] - 1] = a[(int) indices[i] - 1] + incrementValues[i]; // Warning: indices are 1-based (Matlab style)
  }
}
