function [video width height nFrames] = loadVideo(inputFileName, frameIds)
%LOADVIDEO Load specified frames (vector of 1-based indices) of an RGB video. Out-of-range frame IDs are ignored. The output 'video' is of type 'double', with values in [0, 1], and of dimensions (height x width x nFrames x 3). If the last argument is omitted, all frames are loaded. If the last argument is equal to [], no frames are loaded, but the dimensions of the video are loaded and returned.

%   Author: Damien Teney

inputVideo = VideoReader(inputFileName);

nFrames = inputVideo.NumberOfFrames;
width   = inputVideo.Width;
height  = inputVideo.Height;

if nargin < 2 % Second argument not given: load all frames
  frameIds = 1:nFrames;

elseif isempty(frameIds) % Second argument empty: do not load any frames, just return the dimensions and number of frames in the video
  video = [];
  return;

else % Load specified frames
  frameIds = frameIds(frameIds <= nFrames); % Keep only the valid frame IDs
  nFrames = numel(frameIds);
end

if nFrames <= 0
  video = [];
  return;
end % No frame to load: we can stop here

video = zeros(height, width, nFrames, 3); % Initialize

for i = 1:nFrames
  frameId = frameIds(i);

  if frameId <= inputVideo.NumberOfFrames
    frame = read(inputVideo, frameId);
  else
    frame = zeros(height, width, 3);
  end

  video(1:height, 1:width, i, 1:3) = frame;
end

video = double(video) ./ 255; % Get values in [0, 1]
