function video2 = resizeVideo(video, scaleFactor, method)
%RESIZEVIDEO Resize video.
%   VIDEO2 = RESIZEVIDEO(VIDEO, SCALEFACTOR, METHOD) resize the given video
%   (MxNxnFrames or MxNxnFramesx3 matrix) by the given scale factor.
%
%   SCALEFACTOR > 1 means enlarging
%   SCALEFACTOR < 1 means downsizing
%
%   Alternatively, SCALEFACTOR can be a 2-element vector specifying the
%   height and width of the resized video.
%
%   METHOD specifies the interpolation method, is optional and can be
%   'nearest', 'bilinear', or 'bilinear'.

%   Author: Damien Teney

if nargin < 3 % Argument not given
  method = 'bicubic'; % Set default value
  %method = 'bilinear'; % Set default value
end

if numel(scaleFactor) == 1
  newSize = ceil([size(video, 1) size(video, 2)] * scaleFactor);
else
  newSize = scaleFactor;
end

if size(video, 1) == newSize(1) && size(video, 2) == newSize(2)
  video2 = video;
  return;
end

nFrames = size(video, 3);
video2 = zeros(newSize(1), newSize(2), nFrames, size(video, 4)); % Initialize output
for frameId = 1:nFrames % For each frame
  frame = squeeze(video(:, :, frameId, :));
  frame2 = imresize(frame, scaleFactor, method);
  video2(:, :, frameId, :) = frame2;
end
