function saveVideo(video, outputFileName, range, uncompressed)
%SAVEVIDEO Save a video, given as a (height x width x nFrames x nChannels) matrix, as double in [0, 1].

%   Author: Damien Teney

if nargin >= 3
  if isempty(range)
    range = [min(video(:)) max(video(:))]; % Determine range from values in the video
  else
    assert(numel(range) == 2);
  end
  video = (video - range(1)) ./ (range(2) - range(1));
end % Otherwise assume a range of [0, 1]

% Cap values to [0, 1]
video(:) = max(video(:), 0);
video(:) = min(video(:), 1);

% Initialize the video writer
if nargin < 4
   uncompressed = true; % Missing argument: set default value
   %uncompressed = false; % Missing argument: set default value
end
if uncompressed
  videoFileObject = VideoWriter(outputFileName, 'Uncompressed AVI');
else
  %videoFileObject = VideoWriter(outputFileName, 'MPEG-4');
  videoFileObject = VideoWriter(outputFileName);
end
%videoFileObject.FrameRate = 4;
open(videoFileObject);

nFrames = size(video, 3);
for frameId = 1:nFrames
  outputFrame = squeeze(video(:, :, frameId, :));
  writeVideo(videoFileObject, outputFrame);
end

close(videoFileObject);
