function visualize3DVolume1(data, range)
%VISUALIZE3DVOLUME displays 3D data one slice (along the 3rd dimension) at a time, with a slider to select other slices.

% Author: Frederick Bryan, modified by Damien Teney

  assert(ndims(data) == 3);

  if nargin < 2 || isempty(range)
    % Range of values not given: automatic selection
    range = [min(data(:)) max(data(:))];
    if range(2) == range(1), range = [0 1]; end % Special case (all values == 0)
  end

  figureId = gcf;
  clf(figureId, 'reset');
  figure(figureId); 

  printf('\tDimensions: %ux%u\n', size(data, 1), size(data, 2));
  printf('\tRange of values: %.2f .. %.2f\n', min(data(:)), max(data(:)));
  printf('\tIntegral: %.2f\n', sum(data(:)));

  % Add a slider
  mid = ceil(size(data, 3) / 2);
  top = size(data, 3);
  step = [1 1] / (top - 1);
  sliderId = uicontrol('Style', 'slider', 'Min', 1, 'Max', top, 'Value', mid, ...
    'SliderStep', step, 'Position', [10 10 150 25], ...
    'Callback', {@changeSlice, figureId, data, range});
  changeSlice(sliderId, [], figureId, data, range);
end

function changeSlice(src, ~, figureId, data, range)
  % Callback function for the slider: display another slice of the data
  sliceId = round(get(src, 'Value'));
  figure(figureId); imshow(data(:, :, sliceId), range);
  title(sprintf('Slice %d / %d', sliceId, size(data, 3)));
  axis off;
end
