function visualize3DVolume2(data, range, thresholds)
%VISUALIZE3DVOLUME Display 3D data as isosurfaces.

% Author: Damien Teney

assert(ndims(data) == 3);

if nargin < 2 || isempty(range)
  % Range of values not given: automatic selection
  range = [min(data(:)) max(data(:))];
  if range(2) == range(1), range = [0 1]; end % Special case (all values == 0)
end


% Rearrange dimensions to display the 'time' (Z) in the depth of the plot
data = permute(data, [3 2 1]);

%figure();
hold on;

% Positive and negative isosurfaces
if nargin < 3 || isempty(thresholds)
  v1 = prctile(abs(data(data < 0)), 95);
  v2 = prctile(abs(data(data > 0)), 95);
else
  v1 = thresholds(1);
  v2 = thresholds(2);
end
p1 = patch(isosurface(data, -v1));
p2 = patch(isosurface(data, +v2));
isonormals(data, p1);
isonormals(data, p2);
set(p1, 'FaceColor', 'blue', 'EdgeColor', 'none'); 
set(p2, 'FaceColor', 'red', 'EdgeColor', 'none'); 

%daspect([1 1 1]);
%view(3);
view(30, 30);
axis vis3d;
%axis square vis3d;
axis equal;
camlight;
lighting phong;
grid on;
xlabel('X');
zlabel('Y');
ylabel('Time');

xlim([1 size(data, 2)]);
ylim([1 size(data, 1)]);
zlim([1 size(data, 3)]);
