function visualize3DVolume3(data, range)
%VISUALIZE3DVOLUME Display 3D data as a concatenation of all slices along the 3rd dimension.

% Author: Damien Teney

assert(ndims(data) == 3);

if nargin < 2 || isempty(range)
  % Range of values not given: automatic selection
  range = [min(data(:)) max(data(:))];
  if range(2) == range(1), range = [0 1]; end % Special case (all values == 0)
end

figureId = gcf;
clf(figureId, 'reset');
figure(figureId); 

nSlices = size(data, 3);
for k = 1:nSlices
  subplottight(1, nSlices, k);
  imshow(data(:, :, k), range);
end

%==========================================================================
function h = subplottight(Ny, Nx, j, margin)
% Similar to subplot, but removes the spacing between axes.

if nargin < 4 
  margin = 0.1;
end

j = j-1;
x = mod(j,Nx)/Nx;
y = (Ny-fix(j/Nx)-1)/Ny;
h = axes('position', [x + margin/Nx ...
                      y + margin/Ny ...
                      1/Nx - 2*margin/Nx ...
                      1/Ny - 2*margin/Ny]);
