function edges = createEdges2(height, width, nFrames, connectivity)
%CREATEEDGES Create list of edges between adjacent nodes; 6, 10, or 26-connectivity in space/time; edge weights are NOT initialized.

%   Author: Damien Teney

if connectivity ~= 6 && connectivity ~= 10 && connectivity ~= 26
  error('Invalid connectivity for creating edges ! Supported values: 6, 10, 26.');
end

% Declare constants
% Column indices in 'edges'
A = 1; B = 2; WEIGHT = 3;

nNodes = height * width * nFrames;
nodeIds = reshape(1:nNodes, [height width nFrames]);

edges = zeros(0, 2);

%--------------------------------------------------------------------------
% Create edges between pixels: same frame
%--------------------------------------------------------------------------
% Same frame: left neighbour
shift = [0 1 0];
nodeIds_shifted = circshift(nodeIds, shift);
nodeIds1 = nodeIds(        1:height, 2:width, 1:nFrames);
nodeIds2 = nodeIds_shifted(1:height, 2:width, 1:nFrames);
edges = [ edges ; nodeIds1(:) nodeIds2(:) ]; % Append rows

% Same frame: upper neighbour
shift = [1 0 0];
nodeIds_shifted = circshift(nodeIds, shift);
nodeIds1 = nodeIds(        2:height, 1:width, 1:nFrames);
nodeIds2 = nodeIds_shifted(2:height, 1:width, 1:nFrames);
edges = [ edges ; nodeIds1(:) nodeIds2(:) ]; % Append rows

if connectivity >= 10
  % Same frame: upper-left neighbour
  shift = [1 1 0];
  nodeIds_shifted = circshift(nodeIds, shift);
  nodeIds1 = nodeIds(        2:height, 2:width, 1:nFrames);
  nodeIds2 = nodeIds_shifted(2:height, 2:width, 1:nFrames);
  edges = [ edges ; nodeIds1(:) nodeIds2(:) ]; % Append rows

  % Same frame: upper-right neighbour
  shift = [1 -1 0];
  nodeIds_shifted = circshift(nodeIds, shift);
  nodeIds1 = nodeIds(        2:height, 1:width-1, 1:nFrames);
  nodeIds2 = nodeIds_shifted(2:height, 1:width-1, 1:nFrames);
  edges = [ edges ; nodeIds1(:) nodeIds2(:) ]; % Append rows
end

%--------------------------------------------------------------------------
% Create edges between pixels: previous frame
%--------------------------------------------------------------------------
% Previous frame: same positon
shift = [0 0 1];
nodeIds_shifted = circshift(nodeIds, shift);
nodeIds1 = nodeIds(        1:height, 1:width, 2:nFrames);
nodeIds2 = nodeIds_shifted(1:height, 1:width, 2:nFrames);
edges = [ edges ; nodeIds1(:) nodeIds2(:) ]; % Append rows

if connectivity >= 26
  % Previous frame: left neighbour
  shift = [0 1 1];
  nodeIds_shifted = circshift(nodeIds, shift);
  nodeIds1 = nodeIds(        1:height, 2:width, 2:nFrames);
  nodeIds2 = nodeIds_shifted(1:height, 2:width, 2:nFrames);
  edges = [ edges ; nodeIds1(:) nodeIds2(:) ]; % Append rows

  % Previous frame: upper neighbour
  shift = [1 0 1];
  nodeIds_shifted = circshift(nodeIds, shift);
  nodeIds1 = nodeIds(        2:height, 1:width, 2:nFrames);
  nodeIds2 = nodeIds_shifted(2:height, 1:width, 2:nFrames);
  edges = [ edges ; nodeIds1(:) nodeIds2(:) ]; % Append rows

  % Previous frame: upper-left neighbour
  shift = [1 1 1];
  nodeIds_shifted = circshift(nodeIds, shift);
  nodeIds1 = nodeIds(        2:height, 2:width, 2:nFrames);
  nodeIds2 = nodeIds_shifted(2:height, 2:width, 2:nFrames);
  edges = [ edges ; nodeIds1(:) nodeIds2(:) ]; % Append rows

  % Previous frame: upper-right neighbour
  shift = [1 -1 1];
  nodeIds_shifted = circshift(nodeIds, shift);
  nodeIds1 = nodeIds(        2:height, 1:width-1, 2:nFrames);
  nodeIds2 = nodeIds_shifted(2:height, 1:width-1, 2:nFrames);
  edges = [ edges ; nodeIds1(:) nodeIds2(:) ]; % Append rows
end

edges(:, WEIGHT) = 0; % Add (empty) column for edge weights
