function [nodes, nodesParents, pixelAssignment] = createNodes(height, width, nFrames, isFirstBlock, isLastBlock)
%CREATENODES Create list of nodes; one node per pixel.

%   Author: Damien Teney

% Declare constants
% Column indices in 'nodes'
MST_WEIGHT = 1; SIZE = 2; RANK = 3; VARIANCE = 4; COLOR = 5; MIN_FRAME = 8; MAX_FRAME = 9; MIN_X = 10; MAX_X = 11; MIN_Y = 12; MAX_Y = 13; HISTOGRAMS = 14;

nNodes = height * width * nFrames;
nodes = zeros(nNodes, HISTOGRAMS - 1); % Do not create columns for the histograms yet, we will add those later (in another function)
nodes(:, COLOR : (COLOR + 2)) = rand(nNodes, 3); % Assign random colors to each node (will become the colors used to visualize the segments)

% Mark the nodes in the overlap region (first and last frames) as ambiguous
ambiguousNodes = zeros(height, width, nFrames);
if ~isFirstBlock, ambiguousNodes(:, :,   1) = 1; end % First frame of the block (overlap with previous block)
if ~isLastBlock,  ambiguousNodes(:, :, end) = 1; end % Last  frame of the block (overlap with next     block)
nodes(:, VARIANCE) = ambiguousNodes(:);

nodes(:, SIZE) = 1;
nodes(:, MST_WEIGHT) = 0;
nodes(:, RANK) = 0;

frameIds = expandVector(1:nFrames, height * width);
nodes(:, MIN_FRAME) = frameIds(:);
nodes(:, MAX_FRAME) = frameIds(:);
xIndices = repmat(expandVector(1:width, height), [1 1 nFrames]);
nodes(:, MIN_X) = xIndices(:);
nodes(:, MAX_X) = xIndices(:);
yIndices = repmat(1:height, [1 width nFrames]);
nodes(:, MIN_Y) = yIndices(:);
nodes(:, MAX_Y) = yIndices(:);

nodesParents = 1:nNodes; % For each pixel, the index of the root node of its region; initially: root of a node = the node itself

pixelAssignment = reshape(1:nNodes, height, width, nFrames); % Assignment of each pixel of the video to a node
