//=========================================================================
// mergeNodes.h
// Author: Damien Teney
//=========================================================================

// Declare constants (minus 1 each time to get 0-based indices, instead of Matlab's 1-based indices)
// Column indices in 'edges'
#define A           (1-1)
#define B           (2-1)
#define WEIGHT      (3-1)
// Column indices in 'nodes'
#define MST_WEIGHT  (1-1)
#define SIZE        (2-1)
#define RANK        (3-1)
#define VARIANCE    (4-1)
#define COLOR       (5-1)
#define MIN_FRAME   (8-1)
#define MAX_FRAME   (9-1)
#define MIN_X      (10-1)
#define MAX_X      (11-1)
#define MIN_Y      (12-1)
#define MAX_Y      (13-1)
#define HISTOGRAMS (14-1)

#define MIN(a, b) ((a) < (b) ? (a) : (b))
#define MAX(a, b) ((a) > (b) ? (a) : (b))

int findRootNode(int a, double *nodesParents) {
  int a_root;
  a_root = a;
  while ((nodesParents[a_root] - 1) != a_root) // While there is an *other* parent node (parents are "chained")
    a_root = (int) nodesParents[a_root] - 1; // Move up to the parent node
  nodesParents[a] = a_root + 1; // Update for quicker subsequent access (restore 1-based indices !)
  return a_root;
}

void joinNodes(int a, int b, int e, double *nodes, double *nodesParents, int nNodes, int nNodesCols, double *edges, int nEdges) {
  int tmp;
  int k;
  double nodeASize, nodeBSize;
  double edgeWeight;

  // Do "union by rank" for effciency
  // http://en.wikipedia.org/wiki/Disjoint-set_data_structure
  if ( nodes[RANK * nNodes + a] > nodes[RANK * nNodes + b] ) {
    ; // Keep a, nothing to do
  } else if ( nodes[RANK * nNodes + a] < nodes[RANK * nNodes + b] ) {
    tmp = b; b = a; a = tmp; // Keep b: swap a and b (so that the code for the rest of the operations is the same)
  } else { // rank of a == rank of b
    // Keep a, but increment its rank
    nodes[RANK * nNodes + a] = nodes[RANK * nNodes + a] + 1;
  }

  // Join nodes 'a' and 'b', keeping 'a'
  edgeWeight = edges[WEIGHT * nEdges + e];
  nodeASize = nodes[SIZE * nNodes + a];
  nodeBSize = nodes[SIZE * nNodes + b];
  if (nodeASize < 1 || nodeBSize < 1) // Should never happen, error checking for debug
    mexErrMsgTxt("Edge linked to a node of size < 1 !");
  nodes[SIZE      * nNodes + a] = nodeASize + nodeBSize;
  nodes[SIZE      * nNodes + b] = 0; // Mark b as 'deleted'
  nodes[MST_WEIGHT  * nNodes + a] = MAX( MAX(nodes[MST_WEIGHT * nNodes + a], nodes[MST_WEIGHT * nNodes + b]), edgeWeight); // Unnecessary since we process the nodes by increasing weight (see next line)
//nodes[MST_WEIGHT  * nNodes + a] = edgeWeight;
  nodes[MIN_FRAME * nNodes + a] = MIN(nodes[MIN_FRAME * nNodes + a], nodes[MIN_FRAME * nNodes + b]);
  nodes[MAX_FRAME * nNodes + a] = MAX(nodes[MAX_FRAME * nNodes + a], nodes[MAX_FRAME * nNodes + b]);
  nodes[MIN_X     * nNodes + a] = MIN(nodes[MIN_X     * nNodes + a], nodes[MIN_X     * nNodes + b]);
  nodes[MAX_X     * nNodes + a] = MAX(nodes[MAX_X     * nNodes + a], nodes[MAX_X     * nNodes + b]);
  nodes[MIN_Y     * nNodes + a] = MIN(nodes[MIN_Y     * nNodes + a], nodes[MIN_Y     * nNodes + b]);
  nodes[MAX_Y     * nNodes + a] = MAX(nodes[MAX_Y     * nNodes + a], nodes[MAX_Y     * nNodes + b]);
  nodes[VARIANCE  * nNodes + a] = MAX(nodes[VARIANCE  * nNodes + a], nodes[VARIANCE  * nNodes + b]);
  nodesParents[a] = a + 1; // Restore 1-based indices
  nodesParents[b] = a + 1;

  // Merge their histograms
  // (keep the average of the two histograms, weighted by each node's size)
  for (k = HISTOGRAMS; k < nNodesCols; k++) { // For each remaining column of 'nodes'
    nodes[k * nNodes + a] = (nodeASize * nodes[k * nNodes + a]
           + nodeBSize * nodes[k * nNodes + b])
          / (nodeASize + nodeBSize); 
  }

  // Make the edge loop on the node itself (to mark it as useless)
  edges[A * nEdges + e] = a + 1;
  edges[B * nEdges + e] = a + 1;
}
