//=========================================================================
// mergeNodes_fixedNumber(edges, nodes, nodesParents, nEdgesToRemove, minNNodes)
// Author: Damien Teney
//=========================================================================

#include <mex.h>
#include "mergeNodes.h"

//=========================================================================
// Input arguments:
//   edges: nEdges x 3 matrix, type double
//   nodes: nNodes x M matrix, type double
//   nodesParents: nNodes vector, type double
//   nEdgesToRemove: scalar, type double
//   minNNodes: scalar, type double
// Output arguments:
//   none, 'edges', 'nodes', and 'nodesParents' are modified in-place
//=========================================================================
void mexFunction(int nlhs, mxArray *plhs[],
                 int nrhs, const mxArray *prhs[]) {
  // Input
  double *edges, *nodes, *nodesParents;
  int nEdges, nNodes, nNodesCols; // Dimensions of the matrices 'edges' and 'nodes'
  int nNodesActual; // Actual number of nodes, since we do not actually remove rows of the matrix 'nodes', nNodes is constant
  double *tmpPtr, nEdgesToRemove;
  double minNNodes;
  // Misc
  int e, k; // Loop indices
  int a, b, a_root, b_root, tmp; // Node IDs
  double nodeAMeanArea, nodeBMeanArea, nodeASize, nodeBSize;

  // Check the number of arguments
  if (nrhs < 4)
    mexErrMsgTxt ("Wrong number of input arguments !");

  // Get input
  nEdges      = (int) mxGetM(prhs[0]);
  nNodes      = (int) mxGetM(prhs[1]);
  nNodesCols  = (int) mxGetN(prhs[1]);
  nNodesActual = nNodes;
  if ((int) mxGetN(prhs[0]) < 3)
    mexErrMsgTxt ("EDGES must be of dimensions Nx3 !");
  if ((int) mxGetN(prhs[1]) < HISTOGRAMS)
    mexErrMsgTxt ("NODES does not have enough columns !");
  if ((int) mxGetM(prhs[2]) * mxGetN(prhs[2]) != nNodes)
    mexErrMsgTxt ("NODESPARENTS does not have the same number of elements as NODES !");
  if (mxGetClassID(prhs[0]) != mxDOUBLE_CLASS
   || mxGetClassID(prhs[1]) != mxDOUBLE_CLASS
   || mxGetClassID(prhs[2]) != mxDOUBLE_CLASS
   || mxGetClassID(prhs[3]) != mxDOUBLE_CLASS)
    mexErrMsgTxt ("Invalid data types !");
  edges = (double *) mxGetPr(prhs[0]);
  nodes = (double *) mxGetPr(prhs[1]);
  nodesParents = (double *) mxGetPr(prhs[2]);
  tmpPtr = (double *) mxGetPr(prhs[3]); nEdgesToRemove = *tmpPtr;
  tmpPtr = (double *) mxGetPr(prhs[4]); minNNodes = *tmpPtr;

  //-----------------------------------------------------------------------
  // Main code
  //-----------------------------------------------------------------------
  for (e = 0; e < nEdges; e++) { // For each edge
    if (nNodesActual <= minNNodes)
      break;
    if (nEdgesToRemove < 1)
      break;

    a = (int) edges[nEdges * A + e] - 1; // Get 0-based indices
    b = (int) edges[nEdges * B + e] - 1;

    // Find the root node of 'a' and 'b'
    a = findRootNode(a, nodesParents);
    b = findRootNode(b, nodesParents);

    // Update edge info with root nodes
    edges[nEdges * A + e] = a + 1; // Restore 1-based indices
    edges[nEdges * B + e] = b + 1;

    if (a == b)
      continue;

    joinNodes(a, b, e, nodes, nodesParents, nNodes, nNodesCols, edges, nEdges);
    nNodesActual--;
    nEdgesToRemove--;
  } // For each edge
}
