function [edges, nodes, nodesParents, pixelAssignment] = removeChainedLinks(edges, nodes, nodesParents, pixelAssignment)
%REMOVECHAINEDLINKS Update 'nodesParents' (and also 'pixelAssignment' and 'edges') so as to remove chained links, so that each node then points to a root node, or is a root node itself (and points to itself).

%   Author: Damien Teney

% Version with a loop
%%{
for a = 1 : numel(nodesParents) % For each node
  % Find its root node
  rootNode = a;
  while nodesParents(rootNode) ~= rootNode % While there is an *other* parent node (parents are "chained")
    rootNode = nodesParents(rootNode); % Move up to the parent node
  end

  % Update the link (make it direct, i.e. remove the chain)
  nodesParents(a) = rootNode;
end
%}

% Vectorized version
%{
% Find the root nodes
rootNodes = nodesParents(:);
while any(nodesParents(rootNodes) ~= rootNodes) % While there is at least one node that has *another* parent node (parents are "chained")
  rootNodes = nodesParents(rootNodes); % Move up to the parent nodes
end

% Update the links (make them direct, i.e. remove the chains)
nodesParents(:) = rootNodes(:);
%}

% Update the other data structures, so that edges only exist between *root* nodes, and pixels are only assigned to *root* nodes
if ~isempty(edges)
  A = 1; B = 2; WEIGHT = 3; % Column indices in 'edges'
  edges(:, A) = nodesParents(edges(:, A));
  edges(:, B) = nodesParents(edges(:, B));
end
if ~isempty(pixelAssignment)
  pixelAssignment(:) = nodesParents(pixelAssignment(:));
end
