function edges = removeDuplicateEdges(edges, nodesParents)
%REMOVEDUPLICATEEDGES Removing duplicates from the list of edges.
% Warning: the data structure 'nodesParents' should not contain links at this point, they should first have been removed with REMOVECHAINEDLINKS().

%   Author: Damien Teney

% Declare constants
A = 1; B = 2; WEIGHT = 3; % Column indices in 'edges'

if size(edges, 1) == 0, return; end % Empty list of edges: nothing to do

% Update edge info with root nodes (there are no chained links in 'nodesParents', but edges are between arbitrary nodes: we replace these arbitrary nodes by their "root" parent)
edges(:, A) = nodesParents(edges(:, A));
edges(:, B) = nodesParents(edges(:, B));

%  Debug: Check that we now have indeed edges only between root nodes
%assert(all(edges(:, A) == nodesParents(edges(:, A)))); % Checking fails if we did not call updateChainedLinks() before
%assert(all(edges(:, B) == nodesParents(edges(:, B))));

% Remove edges looping on one node
edges = edges((edges(:, A) ~= edges(:, B)), :);

% Reorder a and b with (a <= b), so that we can easily detect duplicate edges
edgesToSwap = edges(:, A) > edges(:, B);
tmp = edges(edgesToSwap, A);
edges(edgesToSwap, A) = edges(edgesToSwap, B);
edges(edgesToSwap, B) = tmp;

%assert(all(edges(:, A) <= edges(:, B))); % Debug check

[trash uniqueEdges] = unique(edges(:, 1:2), 'rows'); % Remove duplicate edges
edges = edges(uniqueEdges, :);
