function [edges, nodes, nodesParents, pixelAssignment] = removeUnusedNodes(edges, nodes, nodesParents, pixelAssignment)
%REMOVEUNUSEDNODES Compact the list of nodes: only keep root nodes.

%   Author: Damien Teney

% Declare constants
% Column indices in 'edges'
A = 1; B = 2; WEIGHT = 3;
% Column indices in 'nodes'
MST_WEIGHT = 1; SIZE = 2; RANK = 3; VARIANCE = 4; COLOR = 5; MIN_FRAME = 8; MAX_FRAME = 9; MIN_X = 10; MAX_X = 11; MIN_Y = 12; MAX_Y = 13; HISTOGRAMS = 14;

assert(all(nodesParents == nodesParents(nodesParents(:)))); % Make sure that they are no chains in nodesParents, i.e. each node is a root node, or has a direct link to a root node

nNodes = size(nodes, 1); % Old number of nodes
rootNodes = (nodesParents(:) == (1 : nNodes)'); % Binary vector: true for each node that is a root node (and that we want to keep)
nNodes2 = nnz(rootNodes); % New number of nodes

nodeIdsMap( rootNodes) = 1 : nNodes2; % For each old root node, its new ID in the (compacted) list of nodes
nodeIdsMap(~rootNodes) = 0; % Those elements should not be used, but it's convenient to set them to 0 for debug (will make the code crash in case of access to those elements)

% Compact / update the data structures
if ~isempty(edges) % If there are edges remaining (there may be only 1 node left, in which case there are no more edges)
  edges(:, A) = nodeIdsMap(edges(:, A)); % Replace by new node IDs
  edges(:, B) = nodeIdsMap(edges(:, B)); % Replace by new node IDs
end
nodes = nodes(rootNodes, :); % Remove unused entries
nodesParents = nodesParents(rootNodes); % Remove unused entries
nodesParents(:) = nodeIdsMap(nodesParents(:)); % Replace by new node IDs
pixelAssignment(:) = nodeIdsMap(pixelAssignment(:)); % Replace by new node IDs

assert(all(nodes(:, SIZE) > 0)); % Debug check
